@@ -1,29 +1,26 @@
-use super::{
-    cargo, git,
-    utils::{names_and_versions, package_by_id, package_eq_dependency},
-    Context, Options,
-};
-use anyhow::bail;
-use cargo_metadata::{Metadata, Package};
-use git_repository::hash::ObjectId;
-use semver::{Op, Version, VersionReq};
 use std::{collections::BTreeMap, str::FromStr};
 
-pub(in crate::command::release_impl) fn edit_version_and_fixup_dependent_crates(
+use anyhow::bail;
+use cargo_metadata::{Metadata, Package};
+use semver::{Op, Version, VersionReq};
+
+use super::{
+    cargo, git,
+    utils::{names_and_versions, package_by_id, package_eq_dependency, will},
+    Context, Oid, Options,
+};
+
+pub(in crate::command::release_impl) fn edit_version_and_fixup_dependent_crates<'repo>(
     meta: &Metadata,
     publishees: &[(&Package, String)],
-    empty_commit_possible: bool,
     Options {
         verbose,
         dry_run,
-        allow_dirty,
+        skip_publish,
         ..
     }: Options,
-    ctx: &Context,
-) -> anyhow::Result<ObjectId> {
-    if !allow_dirty {
-        git::assure_clean_working_tree()?;
-    }
+    ctx: &'repo Context,
+) -> anyhow::Result<Option<Oid<'repo>>> {
     let mut locks_by_manifest_path = BTreeMap::new();
     for (publishee, _) in publishees {
         let lock = git_lock::File::acquire_to_update_resource(
@@ -58,11 +55,12 @@ pub(in crate::command::release_impl) fn edit_version_and_fixup_dependent_crates(
         packages_to_fix.push(package_to_fix);
     }
 
+    let mut made_change = false;
     for (publishee, new_version) in publishees {
         let mut lock = locks_by_manifest_path
             .get_mut(&publishee.manifest_path)
             .expect("lock available");
-        set_version_and_update_package_dependency(
+        made_change |= set_version_and_update_package_dependency(
             publishee,
             Some(&new_version.to_string()),
             publishees,
@@ -75,25 +73,27 @@ pub(in crate::command::release_impl) fn edit_version_and_fixup_dependent_crates(
         let mut lock = locks_by_manifest_path
             .get_mut(&package_to_update.manifest_path)
             .expect("lock written once");
-        set_version_and_update_package_dependency(package_to_update, None, publishees, &mut lock, verbose)?;
+        made_change |=
+            set_version_and_update_package_dependency(package_to_update, None, publishees, &mut lock, verbose)?;
     }
 
-    let message = format!("Release {}", names_and_versions(publishees));
-    if dry_run {
-        if verbose {
-            log::info!("WOULD commit changes to manifests with {:?}", message);
-        }
-        Ok(ObjectId::null_sha1())
-    } else {
-        log::info!("Will persist changes to manifests");
+    let message = format!(
+        "{} {}",
+        if skip_publish { "Bump" } else { "Release" },
+        names_and_versions(publishees)
+    );
+    if verbose {
+        log::info!("{} persist changes to manifests with: {:?}", will(dry_run), message);
+    }
+    if !dry_run {
         for manifest_lock in locks_by_manifest_path.into_values() {
             manifest_lock.commit()?;
         }
         // This is dangerous as incompatibilities can happen here, leaving the working tree dirty.
         // For now we leave it that way without auto-restoring originals to facilitate debugging.
         cargo::refresh_lock_file()?;
-        git::commit_changes(message, empty_commit_possible, ctx)
     }
+    git::commit_changes(message, verbose, dry_run, !made_change, ctx)
 }
 
 fn set_version_and_update_package_dependency(
@@ -102,7 +102,7 @@ fn set_version_and_update_package_dependency(
     publishees: &[(&Package, String)],
     mut out: impl std::io::Write,
     verbose: bool,
-) -> anyhow::Result<()> {
+) -> anyhow::Result<bool> {
     let manifest = std::fs::read_to_string(&package_to_update.manifest_path)?;
     let mut doc = toml_edit::Document::from_str(&manifest)?;
 
@@ -158,7 +158,8 @@ fn set_version_and_update_package_dependency(
             }
         }
     }
-    out.write_all(doc.to_string_in_original_order().as_bytes())?;
+    let new_manifest = doc.to_string_in_original_order();
+    out.write_all(new_manifest.as_bytes())?;
 
-    Ok(())
+    Ok(manifest != new_manifest)
 }
