@@ -1,28 +1,28 @@
-use crate::command::release::Options;
+use std::collections::BTreeSet;
+
 use anyhow::bail;
 use cargo_metadata::{camino::Utf8PathBuf, Dependency, DependencyKind, Metadata, Package, Version};
-use git_repository::{refs::packed, Repository};
-use std::collections::BTreeSet;
+
+use crate::command::release::Options;
 
 mod utils;
 use crates_index::Index;
-use git_repository::hash::ObjectId;
 use utils::{
-    bump_spec_may_cause_empty_commits, is_dependency_with_version_requirement, is_workspace_member, names_and_versions,
-    package_by_id, package_by_name, package_eq_dependency, package_for_dependency, tag_name_for, will,
-    workspace_package_by_id,
+    is_dependency_with_version_requirement, is_workspace_member, names_and_versions, package_by_id, package_by_name,
+    package_eq_dependency, package_for_dependency, tag_name_for, will, workspace_package_by_id,
 };
 
 mod cargo;
 mod git;
 mod manifest;
 
+type Oid<'repo> = git_repository::easy::Oid<'repo, git_repository::Easy>;
+
 pub(in crate::command::release_impl) struct Context {
     root: Utf8PathBuf,
     meta: Metadata,
-    repo: Repository,
-    packed_refs: Option<packed::Buffer>,
-    index: Index,
+    git_easy: git_repository::Easy,
+    crates_index: Index,
     crate_names: Vec<String>,
     bump: String,
     bump_dependencies: String,
@@ -33,14 +33,12 @@ impl Context {
         let meta = cargo_metadata::MetadataCommand::new().exec()?;
         let root = meta.workspace_root.clone();
         let repo = git_repository::discover(&root)?;
-        let packed_refs = repo.refs.packed()?;
         let index = Index::new_cargo_default();
         Ok(Context {
             root,
-            repo,
+            git_easy: repo.into(),
             meta,
-            packed_refs,
-            index,
+            crates_index: index,
             crate_names,
             bump,
             bump_dependencies,
@@ -56,24 +54,31 @@ pub fn release(options: Options, crates: Vec<String>, bump: String, bump_depende
     }
     let mut ctx = Context::new(crates, bump, bump_dependencies)?;
     if options.update_crates_index {
-        log::info!("Updating crates-io index at '{}'", ctx.index.path().display());
-        ctx.index.update()?;
+        log::info!("Updating crates-io index at '{}'", ctx.crates_index.path().display());
+        ctx.crates_index.update()?;
     } else if !options.no_bump_on_demand {
         log::warn!(
             "Consider running with --update-crates-index to assure bumping on demand uses the latest information"
         );
     }
-    if !ctx.index.exists() {
+    if !ctx.crates_index.exists() {
         log::warn!("Crates.io index doesn't exist. Consider using --update-crates-index to help determining if release versions are published already");
     }
 
     if ctx.crate_names.is_empty() {
-        let crate_name = std::env::current_dir()?
+        let current_dir = std::env::current_dir()?;
+        let manifest = current_dir.join("Cargo.toml");
+        let dir_name = current_dir
             .file_name()
             .expect("a valid directory with a name")
             .to_str()
-            .expect("directory is UTF8 representable")
-            .to_owned();
+            .expect("directory is UTF8 representable");
+        let crate_name = if manifest.is_file() {
+            let manifest = cargo_toml::Manifest::from_path(manifest)?;
+            manifest.package.map_or(dir_name.to_owned(), |p| p.name)
+        } else {
+            dir_name.to_owned()
+        };
         log::warn!(
             "Using '{}' as crate name as no one was provided. Specify one if this isn't correct",
             crate_name
@@ -86,11 +91,16 @@ pub fn release(options: Options, crates: Vec<String>, bump: String, bump_depende
 
 fn release_depth_first(ctx: Context, options: Options) -> anyhow::Result<()> {
     let meta = &ctx.meta;
-    let changed_crate_names_to_publish =
-        traverse_dependencies_and_find_crates_for_publishing(meta, &ctx.crate_names, &ctx, options)?;
+    let changed_crate_names_to_publish = if options.skip_dependencies {
+        ctx.crate_names.clone()
+    } else {
+        traverse_dependencies_and_find_crates_for_publishing(meta, &ctx.crate_names, &ctx, options)?
+    };
 
     let crates_to_publish_together = resolve_cycles_with_publish_group(meta, &changed_crate_names_to_publish, options)?;
 
+    assure_working_tree_is_unchanged(options)?;
+
     for publishee_name in changed_crate_names_to_publish
         .iter()
         .filter(|n| !crates_to_publish_together.contains(n))
@@ -98,7 +108,7 @@ fn release_depth_first(ctx: Context, options: Options) -> anyhow::Result<()> {
         let publishee = package_by_name(meta, publishee_name)?;
 
         let (new_version, commit_id) = perform_single_release(meta, publishee, options, &ctx)?;
-        let tag_name = git::create_version_tag(publishee, &new_version, commit_id, &ctx.repo, options)?;
+        let tag_name = git::create_version_tag(publishee, &new_version, commit_id, &ctx, options)?;
         git::push_tags_and_head(tag_name, options)?;
     }
 
@@ -109,6 +119,19 @@ fn release_depth_first(ctx: Context, options: Options) -> anyhow::Result<()> {
     Ok(())
 }
 
+fn assure_working_tree_is_unchanged(options: Options) -> anyhow::Result<()> {
+    if !options.allow_dirty {
+        if let Err(err) = git::assure_clean_working_tree() {
+            if options.dry_run {
+                log::warn!("The working tree has changes which will prevent a release with --execute unless --allow-dirty is also specified. The latter isn't recommended.")
+            } else {
+                return Err(err);
+            }
+        }
+    }
+    Ok(())
+}
+
 fn perforrm_multi_version_release(
     ctx: &Context,
     options: Options,
@@ -135,13 +158,7 @@ fn perforrm_multi_version_release(
         names_and_versions(&crates_to_publish_together)
     );
 
-    let commit_id = manifest::edit_version_and_fixup_dependent_crates(
-        meta,
-        &crates_to_publish_together,
-        bump_spec_may_cause_empty_commits(&ctx.bump) && bump_spec_may_cause_empty_commits(&ctx.bump_dependencies),
-        options,
-        ctx,
-    )?;
+    let commit_id = manifest::edit_version_and_fixup_dependent_crates(meta, &crates_to_publish_together, options, ctx)?;
 
     crates_to_publish_together.reverse();
     let mut tag_names = Vec::new();
@@ -152,7 +169,7 @@ fn perforrm_multi_version_release(
             .collect();
 
         cargo::publish_crate(publishee, &unpublished_crates, options)?;
-        if let Some(tag_name) = git::create_version_tag(publishee, &new_version, commit_id, &ctx.repo, options)? {
+        if let Some(tag_name) = git::create_version_tag(publishee, &new_version, commit_id.clone(), ctx, options)? {
             tag_names.push(tag_name);
         };
     }
@@ -290,11 +307,13 @@ fn depth_first_traversal(
         )?;
         if git::has_changed_since_last_release(dep_package, ctx, verbose)? {
             if dep_package.version.major == 0 || allow_auto_publish_of_stable_crates {
-                log::info!(
-                    "Adding {} v{} to set of published crates as it changed since last release",
-                    dep_package.name,
-                    dep_package.version
-                );
+                if verbose {
+                    log::info!(
+                        "Adding {} v{} to set of published crates as it changed since last release",
+                        dep_package.name,
+                        dep_package.version
+                    );
+                }
                 changed_crate_names_to_publish.push(dependency.name.clone());
             } else {
                 log::warn!(
@@ -425,12 +444,12 @@ fn select_publishee_bump_spec<'a>(name: &String, ctx: &'a Context) -> &'a str {
     }
 }
 
-fn perform_single_release(
+fn perform_single_release<'repo>(
     meta: &Metadata,
     publishee: &Package,
     options: Options,
-    ctx: &Context,
-) -> anyhow::Result<(String, ObjectId)> {
+    ctx: &'repo Context,
+) -> anyhow::Result<(String, Option<Oid<'repo>>)> {
     let bump_spec = select_publishee_bump_spec(&publishee.name, ctx);
     let new_version = bump_to_valid_version(publishee, bump_spec, ctx, options.no_bump_on_demand)?;
     log::info!(
@@ -440,13 +459,8 @@ fn perform_single_release(
         new_version
     );
     let new_version = new_version.to_string();
-    let commit_id = manifest::edit_version_and_fixup_dependent_crates(
-        meta,
-        &[(publishee, new_version.clone())],
-        bump_spec_may_cause_empty_commits(bump_spec),
-        options,
-        ctx,
-    )?;
+    let commit_id =
+        manifest::edit_version_and_fixup_dependent_crates(meta, &[(publishee, new_version.clone())], options, ctx)?;
     cargo::publish_crate(publishee, &[], options)?;
     Ok((new_version, commit_id))
 }
@@ -464,7 +478,7 @@ fn bump_to_valid_version(
         no_bump_on_demand: bool,
     ) -> anyhow::Result<Version> {
         let bump_on_demand = !no_bump_on_demand;
-        match ctx.index.crate_(&publishee.name) {
+        match ctx.crates_index.crate_(&publishee.name) {
             Some(existing_release) => {
                 let existing_version = semver::Version::parse(existing_release.latest_version().version())?;
                 if existing_version >= new_version {
@@ -476,12 +490,22 @@ fn bump_to_valid_version(
                 );
                 }
                 if bump_on_demand && publishee.version > existing_version {
-                    log::info!(
-                        "Using unpublished version {} of crate {} instead of bumped version {}",
-                        publishee.version,
-                        publishee.name,
-                        new_version
-                    );
+                    if new_version > publishee.version {
+                        log::info!(
+                            "Using manifest version {} of crate {} instead of new version {} as it is sufficient to succeed latest published version {}.",
+                            publishee.version,
+                            publishee.name,
+                            new_version,
+                            existing_version
+                        );
+                    } else {
+                        log::info!(
+                            "Using manifest version {} of crate {} as it is sufficient to succeed latest published version {}.",
+                            publishee.version,
+                            publishee.name,
+                            existing_version
+                        );
+                    }
                     new_version = publishee.version.clone();
                 }
             }
@@ -499,7 +523,7 @@ fn bump_to_valid_version(
         };
         Ok(new_version)
     }
-    use semver::{BuildMetadata, Prerelease};
+    use semver::Prerelease;
 
     let mut v = publishee.version.clone();
     match bump_spec {
@@ -507,18 +531,15 @@ fn bump_to_valid_version(
             v.major += 1;
             v.minor = 0;
             v.patch = 0;
-            v.build = BuildMetadata::EMPTY;
             v.pre = Prerelease::EMPTY;
         }
         "minor" => {
             v.minor += 1;
             v.patch = 0;
-            v.build = BuildMetadata::EMPTY;
             v.pre = Prerelease::EMPTY;
         }
         "patch" => {
             v.patch += 1;
-            v.build = BuildMetadata::EMPTY;
             v.pre = Prerelease::EMPTY;
         }
         "keep" => {}
